package com.persagy.apm.alarmservice.group.alarmrule.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.base.CaseFormat;
import com.google.gson.JsonArray;
import com.google.gson.JsonObject;
import com.persagy.apm.alarmservice.group.alarmrule.controller.GroupAlarmRuleController;
import com.persagy.apm.alarmservice.group.alarmrule.dao.GroupAlarmRuleMapper;
import com.persagy.apm.alarmservice.group.alarmrule.model.ConvertGroupAlarmRuleTool;
import com.persagy.apm.alarmservice.group.alarmrule.model.GroupAlarmRule;
import com.persagy.apm.alarmservice.group.alarmrule.model.dto.*;
import com.persagy.apm.alarmservice.group.alarmrule.service.IGroupAlarmRuleService;
import com.persagy.apm.alarmservice.group.management.model.equip.dto.QueryEquipDTO;
import com.persagy.apm.alarmservice.group.management.model.equip.vo.EquipItem;
import com.persagy.apm.alarmservice.group.management.service.impl.GroupManagementServiceImpl;
import com.persagy.apm.alarmservice.project.alarmcondition.model.dto.AddAlarmConditionDTO;
import com.persagy.apm.alarmservice.project.alarmcondition.model.dto.UpdateAlarmConditionDTO;
import com.persagy.apm.alarmservice.project.alarmcondition.service.IAlarmConditionService;
import com.persagy.apm.alarmservice.project.alarmcondition.service.impl.AlarmConditionServiceImpl;
import com.persagy.apm.alarmservice.project.alarmrule.service.IProjectAlarmRuleService;
import com.persagy.apm.alarmservice.project.alarmruleobjrel.model.ProjectAlarmRuleObjRel;
import com.persagy.apm.alarmservice.project.alarmruleobjrel.model.dto.AddProjectAlarmRuleObjRelDTO;
import com.persagy.apm.alarmservice.project.alarmruleobjrel.model.vo.ProjectVO;
import com.persagy.apm.alarmservice.project.alarmruleobjrel.service.IProjectAlarmRuleObjRelService;
import com.persagy.apm.common.constant.enums.ValidEnum;
import com.persagy.apm.common.context.AppContext;
import com.persagy.apm.common.context.poems.PoemsContext;
import com.persagy.apm.common.model.dto.Sort;
import com.persagy.apm.energyalarmstarter.alarmdata.feign.DmpResult;
import com.persagy.apm.energyalarmstarter.alarmdata.model.dto.AlarmCondition;
import com.persagy.apm.energyalarmstarter.alarmdata.model.dto.alarmconfig.*;
import com.persagy.apm.energyalarmstarter.alarmdata.model.dto.common.QueryDTO;
import com.persagy.apm.energyalarmstarter.alarmdata.model.vo.AlarmConfigItem;
import com.persagy.apm.energyalarmstarter.alarmdata.service.AlarmConfigServiceImpl;
import org.apache.commons.collections4.ListUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.io.*;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 集团报警规则(GroupAlarmRule) service层
 *
 * @author lixing
 * @version V1.0 2021-09-08 22:35:45
 */
@Service
public class GroupAlarmRuleServiceImpl extends ServiceImpl<GroupAlarmRuleMapper, GroupAlarmRule>
        implements IGroupAlarmRuleService {
    @Autowired
    private IAlarmConditionService alarmConditionService;
    @Autowired
    private IProjectAlarmRuleService projectAlarmRuleService;
    @Autowired
    private AlarmConfigServiceImpl alarmConfigService;
    @Autowired
    private GroupManagementServiceImpl groupManagementService;
    @Autowired
    private IProjectAlarmRuleObjRelService projectAlarmRuleObjRelService;

    /**
     * 创建集团报警规则
     *
     * @return 集团报警规则主键
     * @author lixing
     * @version V1.0 2021-09-08 22:35:45
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public String createGroupAlarmRule(AddGroupAlarmRuleDTO addGroupAlarmRuleDTO) throws Exception {
        //1、给某个设备分组创建集团报警
        GroupAlarmRule groupAlarmRule = ConvertGroupAlarmRuleTool.INSTANCE.convertAddDto2Entity(addGroupAlarmRuleDTO);
        AddAlarmConditionDTO alarmCondition = addGroupAlarmRuleDTO.getAlarmCondition();
        Map<String, Object> conditionMap = alarmConditionService.createAlarmCondition(alarmCondition);
        // 设置默认值
        groupAlarmRule.setAlarmConditionId((String) conditionMap.get("id"));
        setDefaultValue(groupAlarmRule);
        save(groupAlarmRule);
        //2、同步创建项目报警
        List<AddProjectAlarmRuleObjRelDTO> addProjectAlarmRuleObjRelDTOS = projectAlarmRuleService.addProjectAlarmRules(addGroupAlarmRuleDTO);
        //3、同步创建设备报警条件
        BatchAddAlarmConfigDTO batchAddAlarmConfigDTO = new BatchAddAlarmConfigDTO();
        List<AddAlarmConfigDTO> alarmConfigs = new ArrayList<>();
        for (AddProjectAlarmRuleObjRelDTO addProjectAlarmRuleObjRelDTO : addProjectAlarmRuleObjRelDTOS) {
            AddAlarmConfigDTO addAlarmConfigDTO = new AddAlarmConfigDTO();
            String equipId = addProjectAlarmRuleObjRelDTO.getObjId();
            addAlarmConfigDTO.setObjId(equipId);
            addAlarmConfigDTO.setItemCode(addGroupAlarmRuleDTO.getAlarmItemCode());
            addAlarmConfigDTO.setClassCode(addGroupAlarmRuleDTO.getClassCode());
            addAlarmConfigDTO.setLevel(alarmCondition.getAlarmLevel());
            addAlarmConfigDTO.setCategory("Eqdx");
            addAlarmConfigDTO.setOpen(1);
            AlarmCondition condition = new AlarmCondition();
            List<String> infoCodeList = (List<String>) conditionMap.get("infoCode");
            List<String> infoCodeNew = new ArrayList<>();
            for (String infoCode : infoCodeList) {
                String replacement = equipId + "-" + infoCode;
                infoCodeNew.add(infoCode.replaceAll(infoCode, replacement));
            }
            condition.setInfoCode(infoCodeNew);
            String backend = (String) conditionMap.get("backend");
            for (String infoCode : infoCodeList) {
                String replacement = equipId + "-" + infoCode;
                backend = backend.replaceAll(infoCode, replacement);
            }
            condition.setEnd("!" + backend);
            condition.setEndUphold(0);
            condition.setTrigger(backend);
            condition.setTriggerUphold(alarmCondition.getTriggerUphold() * 60);
            condition.setRules((List<String>) conditionMap.get("displayFormula"));
            JSONArray configs= JSONArray.parseArray(JSON.toJSONString(alarmCondition.getFrontend()));
            condition.setConfigs(configs);
            addAlarmConfigDTO.setCondition(condition);
            alarmConfigs.add(addAlarmConfigDTO);
        }
        batchAddAlarmConfigDTO.setAlarmConfigs(alarmConfigs);
        alarmConfigService.batchCreate(batchAddAlarmConfigDTO);
        return groupAlarmRule.getId();
    }

    /**
     * 如果某些字段没有赋值，使用默认的值
     *
     * @param groupAlarmRule 集团报警规则实体
     * @author lixing
     * @version V1.0 2021/3/12 12:29 下午
     */
    private void setDefaultValue(GroupAlarmRule groupAlarmRule) {
        groupAlarmRule.setCreator(PoemsContext.getContext().getUserId());
        groupAlarmRule.setTs(new Timestamp(System.currentTimeMillis()));
    }

    /**
     * 集团报警规则详情
     *
     * @param id 主键
     * @return 部门do类
     * @author lixing
     * @version V1.0 2021-09-08 22:35:45
     */
    @Override
    public GroupAlarmRule queryGroupAlarmRuleDetail(String id) {
        GroupAlarmRule groupAlarmRule = getById(id);
        if (groupAlarmRule == null) {
            throw new IllegalArgumentException("查看GroupAlarmRule详情时发生异常，找不到要查看的记录，id=" + id);
        }
        return groupAlarmRule;
    }

    /**
     * 更新集团报警规则
     *
     * @author lixing
     * @version V1.0 2021-09-08 22:35:45
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateGroupAlarmRule(UpdateGroupAlarmRuleDTO updateGroupAlarmRuleDTO) throws Exception {
        GroupAlarmRule groupAlarmRule = getById(updateGroupAlarmRuleDTO.getId());
        groupAlarmRule = ConvertGroupAlarmRuleTool.INSTANCE.convertUpdateDto2Entity(groupAlarmRule, updateGroupAlarmRuleDTO);
        groupAlarmRule.setModifier(AppContext.getContext().getAccountId());
        updateById(groupAlarmRule);
        //获得报警规则
        UpdateAlarmConditionDTO alarmCondition = updateGroupAlarmRuleDTO.getAlarmCondition();
        //更新报警规则
        alarmConditionService.updateAlarmCondition(alarmCondition);
        if (alarmCondition != null) {
            //同步更新设备报警条件 begin
            List<UpdateAlarmConfigDTO> updateAlarmConfigDTOs = new ArrayList();
            String condition = updateGroupAlarmRuleDTO.getCondition();
            String classCode = updateGroupAlarmRuleDTO.getClassCode();
            List<ProjectVO> projects = projectAlarmRuleService.getProject();
            List<EquipItem> records = new ArrayList<>();
            for (ProjectVO project : projects) {
                QueryEquipDTO queryEquipDTO = new QueryEquipDTO();
                queryEquipDTO.setClassCode(classCode);
                queryEquipDTO.setCondition(condition);
                queryEquipDTO.setProjectId(project.getProjectId());
                List<EquipItem> equipList = groupManagementService.getEquipList(queryEquipDTO);
                records.addAll(equipList);
            }

            Map<String, Object> stringMap = alarmConditionService.frontend2Backend(alarmCondition.getFrontend());
            for (EquipItem record : records) {
                AlarmCondition alarmCondit = new AlarmCondition();
                alarmCondit.setTrigger(alarmCondition.getBackend());
                List<String> infoCodeList = alarmCondition.getInfoCodes();
                alarmCondit.setInfoCode(infoCodeList);
                alarmCondit.setTriggerUphold(alarmCondition.getTriggerUphold());
                alarmCondit.setRules((List<String>) stringMap.get("displayFormula"));
                UpdateAlarmConfigDTO updateAlarmConfigDTO = new UpdateAlarmConfigDTO();
                updateAlarmConfigDTO.setCondition(alarmCondit);
                updateAlarmConfigDTO.setId(record.getId());
                updateAlarmConfigDTO.setLevel(alarmCondition.getAlarmLevel());
                updateAlarmConfigDTOs.add(updateAlarmConfigDTO);
            }
            List<List<UpdateAlarmConfigDTO>> lists = averageAssign(updateAlarmConfigDTOs, 200);
            for (List<UpdateAlarmConfigDTO> list : lists) {
                BatchUpdateAlarmConfigDTO batchUpdateAlarmConfigDTO = new BatchUpdateAlarmConfigDTO();
                batchUpdateAlarmConfigDTO.setAlarmConfigs(list);
                alarmConfigService.batchUpdate(batchUpdateAlarmConfigDTO);
            }
            //同步更新设备报警条件 end
        }

    }

    /**
     * 校验集团报警规则是否可删除
     *
     * @param id 集团报警规则主键
     * @return 集团报警规则do类
     * @author lixing
     * @version V1.0 2021-09-08 22:35:45
     */
    public GroupAlarmRule checkDeletable(String id) {
        if (id == null) {
            throw new IllegalArgumentException("删除GroupAlarmRule时发生异常，主键为空");
        }

        GroupAlarmRule groupAlarmRule = getById(id);

        if (groupAlarmRule == null) {
            throw new IllegalArgumentException("删除GroupAlarmRule时发生异常，找不到要删除的数据，id:" + id);
        }

        return groupAlarmRule;
    }

    /**
     * 删除集团报警规则
     *
     * @param id 主键
     * @author lixing
     * @version V1.0 2021-09-08 22:35:45
     */
    @Override
    public void deleteGroupAlarmRule(String id) {
        // 校验是否可删除
        GroupAlarmRule groupAlarmRule = checkDeletable(id);
        groupAlarmRule.setValid(ValidEnum.FALSE.getType());
        updateById(groupAlarmRule);
    }

    /**
     * 查询集团报警规则
     *
     * @return List<GroupAlarmRule>
     * @author lixing
     * @version V1.0 2021-09-08 22:35:45
     */
    @Override
    public List<GroupAlarmRule> queryGroupAlarmRuleList(QueryGroupAlarmRuleDTO queryGroupAlarmRuleDTO) {
        QueryWrapper<GroupAlarmRule> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(GroupAlarmRule.PROP_VALID, ValidEnum.TRUE.getType());
        // 默认按创建时间倒序排序
        queryWrapper.orderBy(true, false, GroupAlarmRule.PROP_CREATIONTIME);

        if (queryGroupAlarmRuleDTO != null) {

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryGroupAlarmRuleDTO.getGroupId())) {
                queryWrapper.like(GroupAlarmRule.PROP_GROUP_ID, queryGroupAlarmRuleDTO.getGroupId());
            }

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryGroupAlarmRuleDTO.getAlarmItemName())) {
                queryWrapper.like(GroupAlarmRule.PROP_ALARM_ITEM_NAME, queryGroupAlarmRuleDTO.getAlarmItemName());
            }
        }
        return list(queryWrapper);
    }

    /**
     * 分页查询集团报警规则
     *
     * @return IPage<GroupAlarmRule>
     * @author lixing
     * @version V1.0 2021-09-08 22:35:45
     */
    @Override
    public IPage<GroupAlarmRule> pageQueryGroupAlarmRule(PageQueryGroupAlarmRuleDTO pageQueryGroupAlarmRuleDTO) {
        QueryWrapper<GroupAlarmRule> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(GroupAlarmRule.PROP_VALID, ValidEnum.TRUE.getType());
        // 这里认为pageQueryDTO是经过校验的，肯定包含分页信息
        IPage<GroupAlarmRule> pageParam = new Page<>(pageQueryGroupAlarmRuleDTO.getPage(), pageQueryGroupAlarmRuleDTO.getSize(), true);
        // 排序信息
        if (CollectionUtils.isEmpty(pageQueryGroupAlarmRuleDTO.getOrders())) {
            // 默认按创建时间倒序排序
            queryWrapper.orderBy(true, false, GroupAlarmRule.PROP_CREATIONTIME);
        } else {
            List<Sort> orders = pageQueryGroupAlarmRuleDTO.getOrders();
            for (Sort sort : orders) {
                // 将驼峰转换为下划线格式
                sort.setColumn(CaseFormat.LOWER_CAMEL.to(CaseFormat.LOWER_UNDERSCORE, sort.getColumn()));
                queryWrapper.orderBy(true, sort.isAsc(), sort.getColumn());
            }
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryGroupAlarmRuleDTO.getGroupId())) {
            queryWrapper.eq(GroupAlarmRule.PROP_GROUP_ID, pageQueryGroupAlarmRuleDTO.getGroupId());
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryGroupAlarmRuleDTO.getAlarmItemName())) {
            queryWrapper.like(GroupAlarmRule.PROP_ALARM_ITEM_NAME, pageQueryGroupAlarmRuleDTO.getAlarmItemName());
        }

//        // todo 需判断使用like还是eq
//        if (StringUtils.isNotEmpty(pageQueryGroupAlarmRuleDTO.getAlarmConditionId())) {
//            queryWrapper.like(GroupAlarmRule.PROP_ALARM_CONDITION_ID, pageQueryGroupAlarmRuleDTO.getAlarmConditionId());
//        }

        return getBaseMapper().selectPage(pageParam, queryWrapper);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void delete(DeleteGroupAlarmRuleDTO deleteGroupAlarmRuleDTO) throws Exception {
        this.deleteGroupAlarmRule(deleteGroupAlarmRuleDTO.getId());
        alarmConditionService.deleteAlarmCondition(deleteGroupAlarmRuleDTO.getAlarmConditionId());
        projectAlarmRuleObjRelService.deleteBatch();
        //删除设备报警条件 begin
        BatchDeleteAlarmConfigDTO batchDeleteAlarmConfigDTO = new BatchDeleteAlarmConfigDTO();
        List<DeleteAlarmConfigDTO> alarmConfigs = new ArrayList<>();
        String condition = deleteGroupAlarmRuleDTO.getCondition();
        String classCode = deleteGroupAlarmRuleDTO.getClassCode();
        //查询所有设备
        List<EquipItem> records = groupManagementService.getEquipList(condition, classCode);
        List<String> objIds = records.stream().map(EquipItem::getId).collect(Collectors.toList());
        QueryAlarmConfigDTO queryAlarmConfigDTO = new QueryAlarmConfigDTO();
        queryAlarmConfigDTO.setObjId(objIds);
        DmpResult<List<AlarmConfigItem>> query = alarmConfigService.query(queryAlarmConfigDTO);
        //所有设备条件
        List<AlarmConfigItem> data = query.getData();
        //组织删除参数
        data.forEach(alarmConfigItem -> {
            DeleteAlarmConfigDTO deleteAlarmConfigDTO = new DeleteAlarmConfigDTO();
            deleteAlarmConfigDTO.setId(alarmConfigItem.getId());
            alarmConfigs.add(deleteAlarmConfigDTO);
        });
        batchDeleteAlarmConfigDTO.setAlarmConfigs(alarmConfigs);
        alarmConfigService.batchDelete(batchDeleteAlarmConfigDTO);
        //删除设备报警条件 end
    }

    @Override
    public void readExcel() throws IOException {
        InputStream in = GroupAlarmRuleServiceImpl.class.getResourceAsStream(File.separator + "json" + File.separator + "Project.xlsx");
        XSSFWorkbook sheets = new XSSFWorkbook(in);
        Sheet sheet = sheets.getSheet("慧云3.0及3.1版本清单");
//        String path = this.getClass().getClassLoader().getResource("Project.json").getPath();
        OutputStreamWriter osw = new OutputStreamWriter(new FileOutputStream("/Users/xjh/develop/persagy/energy-alarm-service/src/main/resources/json/Project.json"), "UTF-8");
        List<JSONObject> list = new ArrayList();
        for (int i = 1; i < sheet.getPhysicalNumberOfRows() - 1; i++) {
            Row row = sheet.getRow(i);
            JSONObject subObj = new JSONObject();
            subObj.put("projectId", row.getCell(1).toString());
            subObj.put("projectName", row.getCell(3).toString());
            subObj.put("groupCode", "wd");
            list.add(subObj);
        }
        osw.write(JSON.toJSONString(list));
        osw.flush();
        osw.close();
    }

    @Override
    public GroupAlarmRule getGroupAlarmRule(GetGroupAlarmRuleDTO getGroupAlarmRuleDTO) {
        QueryWrapper<GroupAlarmRule> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(GroupAlarmRule.PROP_GROUP_ID, getGroupAlarmRuleDTO.getGroupId());
        queryWrapper.eq(GroupAlarmRule.PROP_ALARM_ITEM_CODE, getGroupAlarmRuleDTO.getAlarmItemCode());
        return getOne(queryWrapper);
    }

    public static <T> List<List<T>> averageAssign(List<T> source, int n) {
        List<List<T>> result = new ArrayList<List<T>>();
        int remainder = source.size() % n;  //(先计算出余数)
        int number = source.size() / n;  //然后是商
        int offset = 0;//偏移量
        for (int i = 0; i < n; i++) {
            List<T> value = null;
            if (remainder > 0) {
                value = source.subList(i * number + offset, (i + 1) * number + offset + 1);
                remainder--;
                offset++;
            } else {
                value = source.subList(i * number + offset, (i + 1) * number + offset);
            }
            result.add(value);
        }
        return result;
    }
}
