package com.persagy.apm.alarmservice.project.alarmrule.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.persagy.apm.alarmservice.common.util.DataUtils;
import com.persagy.apm.alarmservice.group.alarmrule.model.GroupAlarmRule;
import com.persagy.apm.alarmservice.group.alarmrule.model.dto.AddGroupAlarmRuleDTO;
import com.persagy.apm.alarmservice.group.alarmrule.model.dto.QueryGroupAlarmRuleDTO;
import com.persagy.apm.alarmservice.group.alarmrule.service.IGroupAlarmRuleService;
import com.persagy.apm.alarmservice.group.management.model.equip.dto.QueryEquipDTO;
import com.persagy.apm.alarmservice.group.management.model.equip.vo.EquipItem;
import com.persagy.apm.alarmservice.group.management.model.equip.vo.EquipListItem;
import com.persagy.apm.alarmservice.group.management.service.impl.GroupManagementServiceImpl;
import com.persagy.apm.alarmservice.project.alarmcondition.model.dto.AddAlarmConditionDTO;
import com.persagy.apm.alarmservice.project.alarmcondition.service.IAlarmConditionService;
import com.persagy.apm.alarmservice.project.alarmrule.dao.ProjectAlarmRuleMapper;
import com.persagy.apm.alarmservice.project.alarmrule.model.ConvertProjectAlarmRuleTool;
import com.persagy.apm.alarmservice.project.alarmrule.model.ProjectAlarmRule;
import com.persagy.apm.alarmservice.project.alarmrule.model.dto.AddProjectAlarmRuleDTO;
import com.persagy.apm.alarmservice.project.alarmrule.model.dto.PageQueryProjectAlarmRuleDTO;
import com.persagy.apm.alarmservice.project.alarmrule.model.dto.QueryProjectAlarmRuleDTO;
import com.persagy.apm.alarmservice.project.alarmrule.model.dto.UpdateProjectAlarmRuleDTO;
import com.persagy.apm.alarmservice.project.alarmrule.service.IProjectAlarmRuleService;
import com.persagy.apm.alarmservice.project.alarmruleobjrel.model.dto.AddProjectAlarmRuleObjRelDTO;
import com.persagy.apm.alarmservice.project.alarmruleobjrel.model.dto.QueryRuleObjRelDTO;
import com.persagy.apm.alarmservice.project.alarmruleobjrel.model.vo.ProjectVO;
import com.persagy.apm.alarmservice.project.alarmruleobjrel.service.IProjectAlarmRuleObjRelService;
import com.persagy.apm.common.constant.enums.ValidEnum;
import com.persagy.apm.common.context.AppContext;
import com.persagy.apm.common.context.poems.PoemsContext;
import com.persagy.apm.energyalarmstarter.alarmdata.model.dto.AlarmCondition;
import com.persagy.apm.energyalarmstarter.alarmdata.model.dto.alarmconfig.AddAlarmConfigDTO;
import com.persagy.apm.energyalarmstarter.alarmdata.model.dto.alarmconfig.BatchAddAlarmConfigDTO;
import com.persagy.apm.energyalarmstarter.alarmdata.model.dto.alarmconfig.BatchUpdateAlarmConfigDTO;
import com.persagy.apm.energyalarmstarter.alarmdata.model.dto.alarmconfig.UpdateAlarmConfigDTO;
import com.persagy.apm.energyalarmstarter.alarmdata.service.AlarmConfigServiceImpl;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static org.apache.commons.lang3.math.NumberUtils.INTEGER_ONE;

/**
 * 项目报警规则(ProjectAlarmRule) service层
 *
 * @author lixing
 * @version V1.0 2021-09-08 22:46:16
 */
@Service
public class ProjectAlarmRuleServiceImpl extends ServiceImpl<ProjectAlarmRuleMapper, ProjectAlarmRule>
        implements IProjectAlarmRuleService {

    @Autowired
    private IProjectAlarmRuleObjRelService projectAlarmRuleObjRelService;
    @Autowired
    private IAlarmConditionService alarmConditionService;
    @Autowired
    private IGroupAlarmRuleService groupAlarmRuleService;
    @Autowired
    private GroupManagementServiceImpl groupManagementService;
    @Autowired
    private AlarmConfigServiceImpl alarmConfigService;

    /**
     * 创建项目报警规则
     *
     * @return 项目报警规则主键
     * @author lixing
     * @version V1.0 2021-09-08 22:46:16
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public String createProjectAlarmRule(AddProjectAlarmRuleDTO addProjectAlarmRuleDTO) throws Exception {
        ProjectAlarmRule projectAlarmRule = ConvertProjectAlarmRuleTool.INSTANCE.convertAddDto2Entity(addProjectAlarmRuleDTO);
        AddAlarmConditionDTO addAlarmConditionDTO = addProjectAlarmRuleDTO.getAlarmCondition();
        if (addProjectAlarmRuleDTO.getBelong() == INTEGER_ONE) {
            //查询对应报警类型的集团规则，将ID添加到项目规则表
            QueryGroupAlarmRuleDTO queryAlarmConditionDTO = new QueryGroupAlarmRuleDTO();
            queryAlarmConditionDTO.setAlarmItemCode(addProjectAlarmRuleDTO.getAlarmItemCode());
            List<GroupAlarmRule> groupAlarmRuleList = groupAlarmRuleService.queryGroupAlarmRuleList(queryAlarmConditionDTO);
            String conditionId = groupAlarmRuleList.get(0).getAlarmConditionId();
            projectAlarmRule.setAlarmConditionId(conditionId);
        } else {
            //新增自定义的规则
            Map<String, Object> map = alarmConditionService.createAlarmCondition(addAlarmConditionDTO);
            projectAlarmRule.setAlarmConditionId((String) map.get("id"));
        }
        //新增关联关系
        projectAlarmRuleObjRelService.createBatchRelation(addProjectAlarmRuleDTO.getAddRelationList());
        // 设置默认值
        projectAlarmRule.setCreationTime(new Date());
        setDefaultValue(projectAlarmRule);
        save(projectAlarmRule);
//        //同步创建设备报警条件
//        BatchAddAlarmConfigDTO batchAddAlarmConfigDTO = new BatchAddAlarmConfigDTO();
//        List<AddAlarmConfigDTO> alarmConfigs = new ArrayList<>();
//        for (AddProjectAlarmRuleObjRelDTO addProjectAlarmRuleObjRelDTO : addProjectAlarmRuleDTO.getAddRelationList()) {
//            AddAlarmConfigDTO addAlarmConfigDTO = new AddAlarmConfigDTO();
//            addAlarmConfigDTO.setObjId(addProjectAlarmRuleObjRelDTO.getEquipId());
//            addAlarmConfigDTO.setItemCode(addProjectAlarmRuleDTO.getAlarmItemCode());
//            addAlarmConfigDTO.setClassCode(addProjectAlarmRuleDTO.getClassCode());
//            addAlarmConfigDTO.setLevel(addAlarmConditionDTO.getAlarmLevel());
//            AlarmCondition condition = new AlarmCondition();
//            condition.setTrigger(addAlarmConditionDTO.getBackend());
//            List<String> infoCodeList = addAlarmConditionDTO.getInfoCodes();
//            condition.setInfoCode(infoCodeList);
//            condition.setTriggerUphold(addAlarmConditionDTO.getTriggerUphold());
//            Map<String, Object> stringMap = alarmConditionService.frontend2Backend(addAlarmConditionDTO.getFrontend());
//            condition.setRules((List<String>) stringMap.get("displayFormula"));
//            addAlarmConfigDTO.setCondition(condition);
//            alarmConfigs.add(addAlarmConfigDTO);
//        }
//        batchAddAlarmConfigDTO.setAlarmConfigs(alarmConfigs);
//        alarmConfigService.batchCreate(batchAddAlarmConfigDTO);

        //同步更新设备报警条件 begin
//        BatchUpdateAlarmConfigDTO batchUpdateAlarmConfigDTO = new BatchUpdateAlarmConfigDTO();
//        List<UpdateAlarmConfigDTO> updateAlarmConfigDTOs = new ArrayList();
//        String condition = addProjectAlarmRuleDTO.getCondition();
//        String classCode = addProjectAlarmRuleDTO.getClassCode();
//        List<GroupManagementVo> records = groupManagementService.getEquipList(condition, classCode);
//        Map<String, Object> stringMap = alarmConditionService.frontend2Backend(addAlarmConditionDTO.getFrontend());
//        for (GroupManagementVo record : records) {
//            AlarmCondition alarmCondit = new AlarmCondition();
//            alarmCondit.setTrigger(alarmCondition.getBackend());
//            List<String> infoCodeList = alarmCondition.getInfoCodes();
//            alarmCondit.setInfoCode(infoCodeList);
//            alarmCondit.setTriggerUphold(alarmCondition.getTriggerUphold());
//            alarmCondit.setRules((List<String>) stringMap.get("displayFormula"));
//            UpdateAlarmConfigDTO updateAlarmConfigDTO = new UpdateAlarmConfigDTO();
//            updateAlarmConfigDTO.setCondition(alarmCondit);
//            updateAlarmConfigDTO.setId(record.getId());
//            updateAlarmConfigDTO.setLevel(alarmCondition.getAlarmLevel());
//            updateAlarmConfigDTOs.add(updateAlarmConfigDTO);
//        }
//        batchUpdateAlarmConfigDTO.setAlarmConfigs(updateAlarmConfigDTOs);
//        alarmConfigService.batchUpdate(batchUpdateAlarmConfigDTO);
        //同步更新设备报警条件 end
        return projectAlarmRule.getId();
    }

    /**
     * 如果某些字段没有赋值，使用默认的值
     *
     * @param projectAlarmRule 项目报警规则实体
     * @author lixing
     * @version V1.0 2021/3/12 12:29 下午
     */
    private void setDefaultValue(ProjectAlarmRule projectAlarmRule) {
        projectAlarmRule.setCreator(AppContext.getContext().getAccountId());
        projectAlarmRule.setModifiedTime(new Date());
        // todo 其他默认的属性

    }

    /**
     * 项目报警规则详情
     *
     * @param id 主键
     * @return 部门do类
     * @author lixing
     * @version V1.0 2021-09-08 22:46:16
     */
    @Override
    public ProjectAlarmRule queryProjectAlarmRuleDetail(String id) {
        ProjectAlarmRule projectAlarmRule = getById(id);
        if (projectAlarmRule == null) {
            throw new IllegalArgumentException("查看ProjectAlarmRule详情时发生异常，找不到要查看的记录，id=" + id);
        }
        return projectAlarmRule;
    }

    /**
     * 更新项目报警规则
     *
     * @author lixing
     * @version V1.0 2021-09-08 22:46:16
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateProjectAlarmRule(UpdateProjectAlarmRuleDTO updateProjectAlarmRuleDTO) throws Exception {
        ProjectAlarmRule projectAlarmRule = getById(updateProjectAlarmRuleDTO.getId());
        projectAlarmRule = ConvertProjectAlarmRuleTool.INSTANCE.convertUpdateDto2Entity(projectAlarmRule, updateProjectAlarmRuleDTO);
        projectAlarmRule.setModifier(AppContext.getContext().getAccountId());
        AddAlarmConditionDTO alarmCondition = updateProjectAlarmRuleDTO.getAlarmCondition();
        if (alarmCondition != null) {
            //查询已有报警
            ProjectAlarmRule projectRule = getById(updateProjectAlarmRuleDTO.getId());
            int belongOld = projectRule.getBelong();
            int belongNew = updateProjectAlarmRuleDTO.getBelong();
            //删除对应项目、报警类型、报警对象的关联
            projectAlarmRuleObjRelService.deleteRelation(updateProjectAlarmRuleDTO.getProjectId());
            if (belongOld > belongNew) {//集团规则->项目自定义规则
                //新增自定义的规则
                Map<String, Object> map = alarmConditionService.createAlarmCondition(alarmCondition);
                projectAlarmRule.setAlarmConditionId((String) map.get("id"));
            } else if (belongOld < belongNew) {//项目自定义规则->集团规则
                String conditionId = getConditionId(updateProjectAlarmRuleDTO.getAlarmItemCode());
                projectAlarmRule.setAlarmConditionId(conditionId);
            } else if (belongOld == 0 && belongNew == 0) {//项目自定义规则->项目自定义规则
                //删除自定义规则，新增自定义的规则
                alarmConditionService.deleteAlarmCondition(updateProjectAlarmRuleDTO.getAlarmConditionId());
                Map<String, Object> map = alarmConditionService.createAlarmCondition(alarmCondition);
                projectAlarmRule.setAlarmConditionId((String) map.get("id"));
            }
            //新增关联关系
            projectAlarmRuleObjRelService.createBatchRelation(updateProjectAlarmRuleDTO.getAddRelationList());

            //同步更新设备报警条件 begin
            BatchUpdateAlarmConfigDTO batchUpdateAlarmConfigDTO = new BatchUpdateAlarmConfigDTO();
            List<UpdateAlarmConfigDTO> updateAlarmConfigDTOs = new ArrayList();
            String condition = updateProjectAlarmRuleDTO.getCondition();
            String classCode = updateProjectAlarmRuleDTO.getClassCode();
            List<EquipItem> records = groupManagementService.getEquipList(condition, classCode);
            Map<String, Object> stringMap = alarmConditionService.frontend2Backend(alarmCondition.getFrontend());
            for (EquipItem record : records) {
                AlarmCondition alarmCondit = new AlarmCondition();
                alarmCondit.setTrigger(alarmCondition.getBackend());
                List<String> infoCodeList = alarmCondition.getInfoCodes();
                alarmCondit.setInfoCode(infoCodeList);
                alarmCondit.setTriggerUphold(alarmCondition.getTriggerUphold());
                alarmCondit.setRules((List<String>) stringMap.get("displayFormula"));
                UpdateAlarmConfigDTO updateAlarmConfigDTO = new UpdateAlarmConfigDTO();
                updateAlarmConfigDTO.setCondition(alarmCondit);
                updateAlarmConfigDTO.setId(record.getId());
                updateAlarmConfigDTO.setLevel(alarmCondition.getAlarmLevel());
                updateAlarmConfigDTOs.add(updateAlarmConfigDTO);
            }
            batchUpdateAlarmConfigDTO.setAlarmConfigs(updateAlarmConfigDTOs);
            alarmConfigService.batchUpdate(batchUpdateAlarmConfigDTO);
            //同步更新设备报警条件 end
        }
        //更新项目报警
        projectAlarmRule.setModifiedTime(new Date());
        projectAlarmRule.setModifier(PoemsContext.getContext().getUserId());
        updateById(projectAlarmRule);


    }

    public String getConditionId(String alarmItemCode) {
        //查询对应报警类型的集团规则，将ID添加到项目规则表
        QueryGroupAlarmRuleDTO queryAlarmConditionDTO = new QueryGroupAlarmRuleDTO();
        queryAlarmConditionDTO.setAlarmItemCode(alarmItemCode);
        List<GroupAlarmRule> groupAlarmRuleList = groupAlarmRuleService.queryGroupAlarmRuleList(queryAlarmConditionDTO);
        String conditionId = groupAlarmRuleList.get(0).getAlarmConditionId();
        return conditionId;
    }

    /**
     * 校验项目报警规则是否可删除
     *
     * @param id 项目报警规则主键
     * @return 项目报警规则do类
     * @author lixing
     * @version V1.0 2021-09-08 22:46:16
     */
    public ProjectAlarmRule checkDeletable(String id) {
        if (id == null) {
            throw new IllegalArgumentException("删除ProjectAlarmRule时发生异常，主键为空");
        }

        ProjectAlarmRule projectAlarmRule = getById(id);

        if (projectAlarmRule == null) {
            throw new IllegalArgumentException("删除ProjectAlarmRule时发生异常，找不到要删除的数据，id:" + id);
        }

        return projectAlarmRule;
    }

    /**
     * 删除项目报警规则
     *
     * @param id 主键
     * @author lixing
     * @version V1.0 2021-09-08 22:46:16
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void deleteProjectAlarmRule(String id) {
        // 校验是否可删除
        ProjectAlarmRule projectAlarmRule = checkDeletable(id);
        projectAlarmRule.setValid(ValidEnum.FALSE.getType());
        updateById(projectAlarmRule);
        //查询是否是自定义报警,是则删除自定义规则
        int belong = projectAlarmRule.getBelong();
        if (belong == 0) {
            alarmConditionService.deleteAlarmCondition(projectAlarmRule.getAlarmConditionId());
        }
        //删除项目报警规则与监测对象的关联
        projectAlarmRuleObjRelService.deleteRelation(projectAlarmRule.getProjectId());
    }

    /**
     * 查询项目报警规则
     *
     * @return List<ProjectAlarmRule>
     * @author lixing
     * @version V1.0 2021-09-08 22:46:16
     */
    @Override
    public List<ProjectAlarmRule> queryProjectAlarmRuleList(QueryProjectAlarmRuleDTO queryProjectAlarmRuleDTO) {
        QueryWrapper<ProjectAlarmRule> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ProjectAlarmRule.PROP_VALID, ValidEnum.TRUE.getType());
        // 默认按创建时间倒序排序
        queryWrapper.orderBy(true, false, ProjectAlarmRule.PROP_CREATIONTIME);

        if (queryProjectAlarmRuleDTO != null) {

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryProjectAlarmRuleDTO.getGroupId())) {
                queryWrapper.like(ProjectAlarmRule.PROP_GROUP_ID, queryProjectAlarmRuleDTO.getGroupId());
            }

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryProjectAlarmRuleDTO.getAlarmItemCode())) {
                queryWrapper.like(ProjectAlarmRule.PROP_ALARM_ITEM_CODE, queryProjectAlarmRuleDTO.getAlarmItemCode());
            }

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryProjectAlarmRuleDTO.getAlarmConditionId())) {
                queryWrapper.like(ProjectAlarmRule.PROP_ALARM_CONDITION_ID, queryProjectAlarmRuleDTO.getAlarmConditionId());
            }

            if (queryProjectAlarmRuleDTO.getBelong() != null) {
                queryWrapper.eq(ProjectAlarmRule.PROP_BELONG, queryProjectAlarmRuleDTO.getBelong());
            }

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryProjectAlarmRuleDTO.getProjectId())) {
                queryWrapper.like(ProjectAlarmRule.PROP_PROJECT_ID, queryProjectAlarmRuleDTO.getProjectId());
            }

        }

        return list(queryWrapper);
    }

    /**
     * 分页查询项目报警规则
     *
     * @return IPage<ProjectAlarmRule>
     * @author lixing
     * @version V1.0 2021-09-08 22:46:16
     */
    @Override
    public IPage<ProjectAlarmRule> pageQueryProjectAlarmRule(PageQueryProjectAlarmRuleDTO
                                                                     pageQueryProjectAlarmRuleDTO) {
        QueryWrapper<ProjectAlarmRule> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ProjectAlarmRule.PROP_VALID, ValidEnum.TRUE.getType());
        // 这里认为pageQueryDTO是经过校验的，肯定包含分页信息
        IPage<ProjectAlarmRule> pageParam = new Page<>(pageQueryProjectAlarmRuleDTO.getPage(), pageQueryProjectAlarmRuleDTO.getSize(), true);
        // 排序信息
//        if(CollectionUtils.isEmpty(pageQueryProjectAlarmRuleDTO.getOrders())){
        // 默认按创建时间倒序排序
        queryWrapper.orderBy(true, false, ProjectAlarmRule.PROP_CREATIONTIME);
        /*}else {
            List<Sort> orders = pageQueryProjectAlarmRuleDTO.getOrders();
            for(Sort sort: orders) {
                // 将驼峰转换为下划线格式
                sort.setColumn(CaseFormat.LOWER_CAMEL.to(CaseFormat.LOWER_UNDERSCORE,sort.getColumn()));
                queryWrapper.orderBy(true, sort.isAsc(), sort.getColumn());
            }
        }*/

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryProjectAlarmRuleDTO.getGroupId())) {
            queryWrapper.eq(ProjectAlarmRule.PROP_GROUP_ID, pageQueryProjectAlarmRuleDTO.getGroupId());
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryProjectAlarmRuleDTO.getAlarmItemCode())) {
            queryWrapper.eq(ProjectAlarmRule.PROP_ALARM_ITEM_CODE, pageQueryProjectAlarmRuleDTO.getAlarmItemCode());
        }

        // todo 需判断使用like还是eq
        /*if (StringUtils.isNotEmpty(pageQueryProjectAlarmRuleDTO.getAlarmConditionId())) {
            queryWrapper.like(ProjectAlarmRule.PROP_ALARM_CONDITION_ID, pageQueryProjectAlarmRuleDTO.getAlarmConditionId());
        }*/

//        if (pageQueryProjectAlarmRuleDTO.getBelong() != null) {
//            queryWrapper.eq(ProjectAlarmRule.PROP_BELONG, pageQueryProjectAlarmRuleDTO.getBelong());
//        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryProjectAlarmRuleDTO.getProjectId())) {
            queryWrapper.eq(ProjectAlarmRule.PROP_PROJECT_ID, pageQueryProjectAlarmRuleDTO.getProjectId());
        }

        return getBaseMapper().selectPage(pageParam, queryWrapper);
    }

    @Override
    public List<AddProjectAlarmRuleObjRelDTO> addProjectAlarmRules(AddGroupAlarmRuleDTO addGroupAlarmRuleDTO) {
        String condition = addGroupAlarmRuleDTO.getCondition();
        String classCode = addGroupAlarmRuleDTO.getClassCode();
        //查询所有项目
        List<ProjectVO> projects = this.getProject();
        List<EquipItem> records = new ArrayList<>();
        for (ProjectVO project : projects) {
            QueryEquipDTO queryEquipDTO = new QueryEquipDTO();
            queryEquipDTO.setProjectId(project.getProjectId());
            queryEquipDTO.setClassCode(classCode);
            queryEquipDTO.setCondition(condition);
            List<EquipItem> record = groupManagementService.getEquipList(queryEquipDTO);
            records.addAll(record);
        }
        //遍历项目，组好对象进行插入
        List<ProjectAlarmRule> projectAlarmRuleList = new ArrayList<>();
        String alarmItemCode = addGroupAlarmRuleDTO.getAlarmItemCode();
        String alarmItemName = addGroupAlarmRuleDTO.getAlarmItemName();
        List<AddProjectAlarmRuleObjRelDTO> addRelationList = new ArrayList<>();
        for (ProjectVO project : projects) {
            ProjectAlarmRule projectAlarmRule = new ProjectAlarmRule();
            projectAlarmRule.setProjectId(project.getProjectId());
            projectAlarmRule.setAlarmItemCode(alarmItemCode);
            projectAlarmRule.setAlarmItemName(alarmItemName);
            projectAlarmRule.setBelong(INTEGER_ONE);
            projectAlarmRule.setGroupId(addGroupAlarmRuleDTO.getGroupId());
            projectAlarmRule.setId(DataUtils.getUUID());
            String conditionId = getConditionId(alarmItemCode);
            projectAlarmRule.setAlarmConditionId(conditionId);
            setDefaultValue(projectAlarmRule);
            projectAlarmRuleList.add(projectAlarmRule);
            //新增关联关系
            for (EquipItem record : records) {
                AddProjectAlarmRuleObjRelDTO addProjectAlarmRuleObjRelDTO = new AddProjectAlarmRuleObjRelDTO();
                addProjectAlarmRuleObjRelDTO.setProjectAlarmRuleId(projectAlarmRule.getId());
                addProjectAlarmRuleObjRelDTO.setProjectId(project.getProjectId());
                addProjectAlarmRuleObjRelDTO.setObjId(record.getId());
                addProjectAlarmRuleObjRelDTO.setObjName(record.getName());
                addProjectAlarmRuleObjRelDTO.setOpen(INTEGER_ONE);
                addRelationList.add(addProjectAlarmRuleObjRelDTO);
            }
        }
        projectAlarmRuleObjRelService.createBatchRelation(addRelationList);
        saveBatch(projectAlarmRuleList);
        return addRelationList;
    }


    @Override
    public List<ProjectVO> getProject() {
        InputStream inputStream = ProjectAlarmRuleServiceImpl.class
                .getResourceAsStream(File.separator + "json" + File.separator + "Project.json");
        if (inputStream == null) {
            return null;
        }
        try {
            String str = IOUtils.toString(inputStream, StandardCharsets.UTF_8);
            return JSONArray.parseArray(str, ProjectVO.class);
        } catch (IOException ioe) {
            throw new RuntimeException("json文件有误", ioe);
        }
    }

    @Override
    public List<String> queryAlarmRule(QueryRuleObjRelDTO queryRuleObjRelDTO) {
        QueryWrapper<ProjectAlarmRule> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ProjectAlarmRule.PROP_GROUP_ID, queryRuleObjRelDTO.getGroupId());
        queryWrapper.eq(ProjectAlarmRule.PROP_ALARM_ITEM_CODE, queryRuleObjRelDTO.getAlarmItemCode());
        List<ProjectAlarmRule> list = list(queryWrapper);
        List<String> idList = list.stream().map(ProjectAlarmRule::getId).collect(Collectors.toList());
        return idList;
    }
}
